<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @link       orionorigin.com
 * @since      1.0.0
 *
 * @package    Vpc_Mva
 * @subpackage Vpc_Mva/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Vpc_Mva
 * @subpackage Vpc_Mva/admin
 * @author     orion <help@orionorigin.com>
 */
class Vpc_Mva_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string $plugin_name The name of this plugin.
	 * @param      string $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
		if ( class_exists( 'vpc' ) ) {
			if ( is_vpc_admin_screen() ) {
				wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/vpc-mva-admin.css', array(), VPC_MVA_VERSION, 'all' );
			}
		}
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		if ( class_exists( 'vpc' ) ) {
			if ( is_vpc_admin_screen() ) {
				wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/vpc-mva-admin.js', array( 'jquery' ), VPC_MVA_VERSION, false );
				?>
		<script >
			var view_notice = "<?php esc_html_e( 'You need to save the data so that the new views are detected', 'vpc-mva' ); ?>";
		</script>
				<?php
			}
		}
	}

	/**
	 * Runs the new version check and upgrade process
	 *
	 * @return \Vpc_Mva_Updater
	 */
	public function get_updater() {
		if ( class_exists( 'vpc' ) ) {
			do_action( 'vpc_before_init_updater' );
			require_once VPC_MVA_DIR . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'updaters' . DIRECTORY_SEPARATOR . 'class-vpc-mva-updater.php';
			$updater = new Vpc_Mva_Updater();
			$updater->init();
			require_once VPC_MVA_DIR . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'updaters' . DIRECTORY_SEPARATOR . 'class-vpc-mva-updating-manager.php';
			$updater->set_update_manager( new Vpc_Mva_Updating_Manager( VPC_MVA_VERSION, $updater->version_url(), VPC_MVA_MAIN_FILE ) );
			do_action( 'vpc_after_init_updater' );
			return $updater;
		}
	}

	/**
	 * Create views columns.
	 *
	 * @param type $options_fields The options fields.
	 * @param type $config_id      The configuration ID.
	 * @return type array
	 */
	public function create_views_columns( $options_fields, $config_id ) {
		$config    = get_post_meta( $config_id, 'vpc-config', true );
		$views_arr = vpc_mva_get_config_views( $config );
		if ( is_array( $views_arr ) ) {
			foreach ( $views_arr as $key => $view ) {
				$options_fields['fields'][] = array(
					'title'     => trim( $view ),
					'name'      => 'view_' . $key,
					'url_name'  => 'image_url_' . trim( $view ),
					'type'      => 'image',
					'set'       => 'Set',
					'class'     => 'views',
					'remove'    => 'Remove',
					'col_class' => 'vpc-view-column',
				);
			}
		}
		return $options_fields;
	}

	/**
	 * Add muultiview global settings.
	 *
	 * @param array $global_settings The global settings.
	 * @return type
	 */
	public function add_global_settings( $global_settings ) {
		$begin = array(
			'type'  => 'sectionbegin',
			'id'    => 'vpc-mva-container',
			'table' => 'options',
		);

		$end = array( 'type' => 'sectionend' );

		$license_key = array(
			'title'   => __( 'Muliple views add on license key', 'vpc' ),
			'name'    => 'vpc-options[purchase-code-multiple-views-add-on]',
			'type'    => 'text',
			'desc'    => __( 'This is the licence key you received after your purchase.<a href=“https://orionorigin.com/guide/how-to-add-my-license/” target=“_blank”>Learn more</a>. <a href=“https://orionorigin.com/my-account/orders/“ target=“blank”>Where can I find my license key</a>?', 'vpc' ),
			'default' => '',
		);

		array_push( $global_settings, $begin );
		array_push( $global_settings, $license_key );
		array_push( $global_settings, $end );
		return $global_settings;
	}

	/**
	 *  Add multiview settings.
	 *
	 * @param array $skin_settings The skin settings.
	 * @return array
	 */
	public function add_multiview_settings( $skin_settings ) {
		$skin_settings[]      = array(
			'type' => 'sectionbegin',
			'id'   => 'vpc-multi-container',
		);
		$multiview_guide_link = html_entity_decode( "<a href='https://orionorigin.com/guide/multiple-views-add-on/' target='_blank'>" . __( 'Learn more', 'vpc-mva' ) . '</a>.' );
		$skin_settings[]      = array(
			'title'   => __( 'Activate Multi views', 'vpc-mva' ),
			'name'    => 'vpc-config[multi-views]',
			'type'    => 'radio',
			'options' => array(
				'Yes' => 'Yes',
				'No'  => 'No',
			),
			'default' => 'No',
			'class'   => 'view_option',
			'desc'    => __( 'Choosing YES lets you add more than one view for the product components you set up within the configuration - ', 'vpc-mva' ) . wp_kses( $multiview_guide_link, get_allowed_tags() ),
		);
		$skin_settings[]      = array(
			'title'             => __( 'Views', 'vpc-mva' ),
			'name'              => 'vpc-config[views]',
			'type'              => 'textarea',
			'class'             => 'viewsNames',
			'desc'              => __( 'Add names for the different views your components will have. Add one name per line.', 'vpc-mva' ),
			'custom_attributes' => array(
				'cols' => 30,
				'rows' => 8,
			),
		);
		$skin_settings[]      = array( 'type' => 'sectionend' );
		return $skin_settings;
	}

	/**
	 * Set default view of each component.
	 *
	 * @param array  $components  Components array.
	 * @param string $config_id  The configuration ID.
	 * @return type
	 */
	public function add_component_default_view( $components, $config_id ) {
		$config               = get_post_meta( $config_id, 'vpc-config', true );
		$init_array           = array( 'none' => 'None' );
		$views                = array_merge( $init_array, vpc_mva_get_config_views( $config ) );
		$multiview_guide_link = html_entity_decode( "<a href='https://orionorigin.com/guide/multiple-views-add-on/' target='_blank'>More</a>." );
		$view_selector        = array(
			'title'     => __( 'Default view', 'vpc-mva' ) . ' (' . wp_kses( $multiview_guide_link, get_allowed_tags() ) . ')',
			'name'      => 'component_focus_view',
			'type'      => 'select',
			'options'   => $views,
			'class'     => 'vpc-component-focus',
			'col_class' => 'vpc-default-view-column',
		);
		array_push( $components['fields'], $view_selector );
		return $components;
	}

	/**
	 * Run db updates requirements
	 */
	public function run_db_updates_requirements() {
		// Checks db structure for v2.0.
		$vpc_mva_db_version = get_option( 'vpc-mva-db-version' );
		if ( empty( $vpc_mva_db_version ) ) {
			$vpc_mva_db_version = 0;
		}
		if ( $vpc_mva_db_version < 2 ) {
			$new_mva_db_version = 2;
			?>
		<div class="updated" id="vpc-mva-updater-container">
			<h2><?php esc_html_e( 'An update is required for the Multiviews add-on database.', 'vpc-mva' ); ?></h2>
			<div>
			<ul>
				<li>
				<h3><?php esc_html_e( 'Multiviews add-on', 'vpc-mva' ); ?></h3>
				<ul>
							<li><?php esc_html_e( 'Henceforth, the first step in creating views is to activate and add them inside the configuration', 'vpc-mva' ); ?></li>
							<li><?php esc_html_e( 'Not to worry, clicking on the “Run Updater” button below will recreate all your previously created views found in your data.', 'vpc-mva' ); ?></li>
				</ul>
				</li>
			</ul>

			<input type="button" value="<?php echo esc_html_e( 'Run the updater', 'vpc-mva' ); ?>" id="vpc-mva-run-updater" class="button button-primary"/>
			<div class="loading" style="display:none;"></div>
			</div>
		</div>
			<?php
		} elseif ( $vpc_mva_db_version < 3 ) {
			$new_mva_db_version = 3;
			?>
		<div class="updated" id="vpc-mva-updater-container">
			<h2><?php esc_html_e( 'An update is required for the Multiviews add-on database.', 'vpc-mva' ); ?></h2>
			<div>
			<ul>
				<li>
				<h3><?php esc_html_e( 'Multiviews add-on', 'vpc-mva' ); ?></h3>
				<ul>
							<li><?php esc_html_e( 'Henceforth, the first step in creating views is to activate and add them inside the configuration', 'vpc-mva' ); ?></li>
							<li><?php esc_html_e( 'Not to worry, clicking on the “Run Updater” button below will recreate all your previously created views found in your data.', 'vpc-mva' ); ?></li>
				</ul>
				</li>
			</ul>
			<input type="button" value="<?php echo esc_html_e( 'Run the updater', 'vpc-mva' ); ?>" id="vpc-mva-run-updater" class="button button-primary"/>
			<div class="loading" style="display:none;"></div>
			</div>
		</div>
			<?php
		}
		if ( $vpc_mva_db_version < 2 || $vpc_mva_db_version < 3 ) {
			?>
		<style>
			#vpc-mva-updater-container
			{
			padding: 3px 17px;
			/*font-size: 13px;*/
			line-height: 36px;
			margin-left: 0px;
			border-left: 5px solid #e14d43 !important;
			}
			#vpc-mva-updater-container.done
			{
			border-color: #7ad03a !important;
			}
			#vpc-run-updater {
			background: #e14d43;
			border-color: #d02a21;
			color: #fff;
			-webkit-box-shadow: inset 0 1px 0 #ec8a85,0 1px 0 rgba(0,0,0,.15);
			box-shadow: inset 0 1px 0 #ec8a85,0 1px 0 rgba(0,0,0,.15);
			text-shadow: none;
			}

			#vpc-run-updater:focus, #vpc-run-updater:hover {
			background: #dd362d;
			border-color: #ba251e;
			color: #fff;
			-webkit-box-shadow: inset 0 1px 0 #e8756f;
			box-shadow: inset 0 1px 0 #e8756f;
			}
			.loading
			{
			background: url("<?php echo esc_url( VPC_URL ); ?>/admin/images/spinner.gif") 10% 10% no-repeat transparent;
			background-size: 111%;
			width: 32px;
			height: 40px;
			display: inline-block;
			}
		</style>
		<script>
			//jQuery('.loading').hide();
			jQuery('#vpc-mva-run-updater').click('click', function () {
			var ajax_url = "<?php echo esc_url( admin_url( 'admin-ajax.php' ) ); ?>";
			if (confirm("It is strongly recommended that you make a backup your database before proceeding. Are you sure you wish to run the updater now")) {
				jQuery('.loading').show();
				jQuery.post(
					ajax_url,
					{
					action: 'mva_run_updater',
					mva_new_db_version: <?php echo esc_html( $new_mva_db_version ); ?>,
					run_updater_nonce: "<?php echo esc_html( wp_create_nonce( 'run_updater' ) ); ?>"
					},
					function (data) {
					;
					jQuery('.loading').hide();
					jQuery('#vpc-mva-updater-container').html(data);
					jQuery('#vpc-mva-updater-container').addClass("done");
					location.reload();
					}
				);
			}

			});
		</script>
			<?php
		}
	}

	/**
	 * Run updater
	 */
	public function run_updater() {
		ob_start();
		check_ajax_referer( 'run_updater', 'run_updater_nonce' );
		if ( isset( $_POST['mva_new_db_version'] ) ) {
			$mva_db_version = sanitize_text_field( wp_unslash( $_POST['mva_new_db_version'] ) );
			if ( 2 === absint( sanitize_text_field( wp_unslash( $_POST['mva_new_db_version'] ) ) ) ) {
				// We migrate the multiple views data.
				$this->migrate_multiview_data();
				esc_html_e( 'Done!', 'vpc' );
				update_option( 'vpc-mva-db-version', 2 );
			} elseif ( 3 === absint( sanitize_text_field( wp_unslash( $_POST['mva_new_db_version'] ) ) ) ) {
				// We migrate the multiple views data.
				$this->migrate_new_multiview_data();
				esc_html_e( 'Done!', 'vpc' );
				update_option( 'vpc-mva-db-version', 3 );
			}
		} else {
			echo esc_html__( 'There is no migration callback available for your actual DB version.', 'vpc-mva' );
		}
		$output = ob_get_contents();
		ob_end_clean();
		echo esc_attr( $output );
		die();
	}

	/**
	 * Migrate multiview data
	 *
	 * @global type $wpdb
	 */
	public function migrate_multiview_data() {
		global $wpdb;
		$sql        = 'select distinct post_id from ' . $wpdb->postmeta . ' where meta_value like "%\"multi-views\";s:3:\"yes\"%"';
		$views_used = array();

		$configs_to_migrate = $wpdb->get_col( $sql ); // phpcs:ignore
		if ( ! empty( $configs_to_migrate ) ) {
			foreach ( $configs_to_migrate as $config_id ) {
				$config_metas = get_post_meta( $config_id, 'vpc-config', true );
				$all_metas    = $config_metas;
				foreach ( $config_metas['components'] as $i => $component ) {
					foreach ( $component['options'] as $j => $option ) {
						if ( isset( $option['view_options'] ) ) {
							foreach ( $option['view_options'] as $view_option ) {
								$views_used[] = array(
									'view'  => $view_option['view'],
									'image' => $view_option['image'],
								);
								$view_name    = $this->get_view_name( $view_option['view'] );
								$all_metas['components'][ $i ]['options'][ $j ][ 'view_' . sanitize_title( $view_name ) ] = $view_option['image'];
							}
						}
					}
				}

				$all_metas = $this->import_old_views_to_news_views( $all_metas, $views_used );

				update_post_meta( $config_id, 'vpc-config', $all_metas );
			}
		}
	}

	/**
	 * Function to migrate multiview data.
	 *
	 * @global type $wpdb
	 */
	public function migrate_new_multiview_data() {
		global $wpdb;
		$sql                = 'select distinct post_id from ' . $wpdb->postmeta . ' where meta_value like "%\"multi-views\";s:3:\"yes\"%"';
		$views_used         = array();
		$configs_to_migrate = $wpdb->get_col( $sql ); // phpcs:ignore
		if ( ! empty( $configs_to_migrate ) ) {
			foreach ( $configs_to_migrate as $config_id ) {
				$config_metas = get_post_meta( $config_id, 'vpc-config', true );
				$views_used   = vpc_mva_get_config_views( $config_metas );
				$all_metas    = $config_metas;
				foreach ( $config_metas['components'] as $i => $component ) {
					foreach ( $component['options'] as $j => $option ) {
						if ( isset( $views_used ) && ! empty( $views_used ) ) {
							foreach ( $views_used as $view_key => $view_value ) {
								if ( isset( $option[ 'view_' . sanitize_title( $view_value ) ] ) && '' !== $option[ 'view_' . sanitize_title( $view_value ) ] ) {
										$all_metas['components'][ $i ]['options'][ $j ][ 'view_' . $view_key ] = $all_metas['components'][ $i ]['options'][ $j ][ 'view_' . sanitize_title( $view_value ) ];
								}
								if ( isset( $option[ 'view_' . sanitize_title( $view_value ) . '_text_component' ] ) && '' !== $option[ 'view_' . sanitize_title( $view_value ) . '_text_component' ] ) {
										$all_metas['components'][ $i ]['options'][ $j ][ 'view_' . $view_key . '_text_component' ] = $all_metas['components'][ $i ]['options'][ $j ][ 'view_' . sanitize_title( $view_value ) . '_text_component' ];
								}
								if ( isset( $option[ 'view_' . sanitize_title( $view_value ) . '_upload_component' ] ) && '' !== $option[ 'view_' . sanitize_title( $view_value ) . '_upload_component' ] ) {
									$all_metas['components'][ $i ]['options'][ $j ][ 'view_' . $view_key . '_upload_component' ] = $all_metas['components'][ $i ]['options'][ $j ][ 'view_' . sanitize_title( $view_value ) . '_upload_component' ];
								}
							}
						}
					}
				}
				update_post_meta( $config_id, 'vpc-config', $all_metas );
			}
		}
	}

	/**
	 * Get view new.
	 *
	 * @param type $view_id The view id.
	 * @return type
	 */
	private function get_view_name( $view_id ) {
		$old_views = get_option( 'vpc-mva-views' );
		$view_id   = (int) $view_id;
		return $old_views[ $view_id ];
	}

	/**
	 * Import old views to news views structures
	 *
	 * @param type $config_metas   Configuration datas.
	 * @param type $views_used     Configuration views.
	 * @return type
	 */
	private function import_old_views_to_news_views( $config_metas, $views_used ) {
		$old_views = get_option( 'vpc-mva-views' );
		$views     = array();
		if ( isset( $config_metas['views'] ) ) {
			$views = vpc_mva_get_config_views( $config_metas );
		}

		if ( is_array( $views_used ) ) {
			foreach ( $views_used as $view_used ) {
				$view = $this->get_view_name( $view_used['view'] );
				if ( ! in_array( $view, $views, true ) ) {
					array_push( $views, $view );
				}
			}
			$views                 = array_unique( $views );
			$views                 = array_filter( $views );
			$views                 = array_values( $views );
			$config_metas['views'] = implode( PHP_EOL, $views );
		}
		return $config_metas;
	}

}
