<?php
/**
 * The public-facing functionality of the plugin.
 *
 * @link       orionorigin.com
 * @since      1.0.0
 *
 * @package    Vpc_Mva
 * @subpackage Vpc_Mva/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Vpc_Mva
 * @subpackage Vpc_Mva/public
 * @author     orion <help@orionorigin.com>
 */
class Vpc_Mva_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;
	/**
	 * All activ views.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $active_views    All activ views.
	 */
	public $active_views;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string $plugin_name       The name of the plugin.
	 * @param      string $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name  = $plugin_name;
		$this->version      = $version;
		$this->active_views = array();
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
		wp_enqueue_style( 'vpc-mva-css', plugin_dir_url( __FILE__ ) . 'css/vpc-mva-public.css', array(), VPC_MVA_VERSION, 'all' );
		wp_enqueue_style( 'vpc-mva-bxslider-css', plugin_dir_url( __FILE__ ) . 'css/jquery.bxslider.min.css', array(), VPC_MVA_VERSION, 'all' );
	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		wp_enqueue_script( 'vpc-mva-js', plugin_dir_url( __FILE__ ) . 'js/vpc-mva-public.js', array( 'jquery' ), VPC_MVA_VERSION, false );
		wp_enqueue_script( 'vpc-mva-bxslider.js', plugin_dir_url( __FILE__ ) . 'js/jquery.bxslider.js', array( 'jquery' ), VPC_MVA_VERSION, false );
	}

	/**
	 * Retrieve the views used by the configurator
	 *
	 * @param type $config        The configuration object.
	 * @param type $product_id    The product ID.
	 * @param type $config_id     The configuration ID.
	 */
	public function retrieve_views_used( $config, $product_id, $config_id ) {
		if ( true === ( isset( $config['multi-views'] ) && filter_var( $config['multi-views'], FILTER_VALIDATE_BOOLEAN ) ) ) {
			$views_arr = vpc_mva_get_config_views( $config );
			foreach ( $config['components'] as $components ) {
				if ( is_array( $components['options'] ) ) {
					foreach ( $components['options'] as $option ) {
						foreach ( $views_arr as $key => $view ) {
							if ( isset( $option[ 'view_' . $key ] ) && ! empty( $option[ 'view_' . $key ] ) ) {
								if ( ! in_array( sanitize_title( trim( $view ) ), $this->active_views, true ) ) {
										$this->active_views[ $key ] = sanitize_title( trim( $view ) );
								}
							}
						}
					}
				}
			}
			$this->active_views = apply_filters( 'vpc_mva_set_active_views', $this->active_views, $config, $product_id, $config_id );
			ksort( $this->active_views );
			?>
		<script>
			var active_views = '<?php echo wp_json_encode( $this->active_views ); ?>';
		</script>
			<?php
		}
	}

	/**
	 * Build multiviews's preview
	 *
	 * @param type $preview_html   The configurator preview output.
	 * @param type $prod_id        The product ID.
	 * @param type $config_id      The configuration ID.
	 * @return type
	 */
	public function build_multiview_preview( $preview_html, $prod_id, $config_id ) {

		$config = vpc_mva_get_config_data( $prod_id );
		if ( true === ( isset( $config['multi-views'] ) && filter_var( $config['multi-views'], FILTER_VALIDATE_BOOLEAN ) ) ) {
			if ( ! get_option( 'vpc-multiple-views-add-on-license-key' ) ) {
				echo "<h2 class='error_msg'>You have not activated your license yet. Please, activate it in order to use Multiple views add on.</h2>";
			} else {
				$preview_html = "<ul class='bxslider'>";
				foreach ( $this->active_views as $key => $view ) {
					$view_name     = sanitize_title( trim( $view ) );
					$preview_html .= '<li class="vpc-preview" id="preview_' . $view_name . '" data-view="' . $key . '" data-view-name="' . $view . '"><div id="vpc-preview' . $key . '" ></div></li>';
				}
				$preview_html .= '</ul>';
				$preview_html .= '<div id="mva-bx-pager" class="bx-wrapper">
        <div class="bx-pager">';
				$i             = 0;
				foreach ( $this->active_views as $key => $view ) {
					$view_name     = apply_filters( 'vpc_mva_view_name', trim( $view ), $key, $view );
					$pager_display = '<div class="bx-pager-item"><a data-slide-index="' . $i . '" class="bx-pager-link" data-view="' . $key . '">' . str_replace( '-', ' ', $view ) . '</a></div>';
					$pager_display = apply_filters( 'vpc_mva_pager_display', $pager_display, $view_name, $key, $view, $i );
					$preview_html .= $pager_display;
					$i ++;
				}
				$preview_html .= '</div>';
				$preview_html .= '</div>';
			}
		}

		return apply_filters( 'vpc_mva_preview_html', $preview_html, $config, $prod_id, $config_id, $this->active_views );
	}

	/**
	 * Set option custom data attributes
	 *
	 * @param string $customs_datas     All custom attributes for options.
	 * @param type   $option            Component's options.
	 * @param type   $component         Component datas.
	 * @param type   $config            The configurations.
	 * @return string
	 */
	public function set_option_custom_data_attributes( $customs_datas, $option, $component, $config ) {
		if ( true === ( isset( $config['multi-views'] ) && filter_var( $config['multi-views'], FILTER_VALIDATE_BOOLEAN ) ) ) {
			$views_arr = vpc_mva_get_config_views( $config );
			foreach ( $views_arr as $key => $view ) {
				if ( isset( $option[ 'view_' . $key ] ) && ! empty( $option[ 'view_' . $key ] ) ) {
					$o_image        = o_get_proper_image_url( $option[ 'view_' . $key ] );
					$customs_datas .= ' data-' . sanitize_title( trim( $view ) ) . "=$o_image";
				}
			}
		}
		return $customs_datas;
	}

	/**
	 * Add multi views option value in global variable
	 *
	 * @param type $datas  The configurations global datas.
	 * @return boolean
	 */
	public function add_multiviews_option( $datas ) {
		$config         = vpc_mva_get_config_data( $datas['product'] );
		$datas['views'] = false;
		if ( true === ( isset( $config['multi-views'] ) && filter_var( $config['multi-views'], FILTER_VALIDATE_BOOLEAN ) ) ) {
			$datas['views'] = true;
		}
		return $datas;
	}

	/**
	 * Set components customs data attributes
	 *
	 * @param string $customs_attributes    All custom attributes for component.
	 * @param type   $product_id            The product ID.
	 * @param type   $component             Component datas.
	 * @return string
	 */
	public function set_component_customs_data_attributes( $customs_attributes, $product_id, $component ) {
		$component_focus_view = get_proper_value( $component, 'component_focus_view', 'none' );
		$config               = vpc_mva_get_config_data( $product_id );
		$views                = vpc_mva_get_config_views( $config );
		$view                 = get_proper_value( $views, $component_focus_view, 'none' );
		//$view_focus           = sanitize_title( $view );
		$view_focus 	 	= array_search($view, $views);
		$customs_attributes  .= " data-component-focus=$view_focus";
		return $customs_attributes;
	}

}
